@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import errno
 import json
 import os
 
@@ -29,6 +30,10 @@ sample_data_path = os.path.join(
 
 distros = ['Ubuntu', 'Debian', 'Fedora', 'RedHat', 'CentOS', 'Gentoo']
 styles = ['hp', 'rax', 'liberty', 'nokey']
+ips = {'hp': '127.0.1.1',
+       'rax': '10.208.169.118',
+       'liberty': '192.0.2.2',
+       'nokey': '127.0.1.1'}
 
 built_scenarios = []
 for distro in distros:
@@ -44,6 +49,7 @@ class TestGlean(base.BaseTestCase):
 
     def setUp(self):
         super(TestGlean, self).setUp()
+        self._resolv_unlinked = False
 
     def _patch_argv(self, args):
         self.useFixture(fixtures.MonkeyPatch('sys.argv', ['.'] + args))
@@ -66,7 +72,21 @@ class TestGlean(base.BaseTestCase):
                     mock_handle.__enter__ = mock.Mock(return_value=mock_handle)
                     mock_handle.__exit__ = mock.Mock()
                     mock_handle.read.return_value = ''
+                    # Test broken symlink handling -- we want to
+                    # unlink the file and write a new one.  Simulate a
+                    # /etc/resolv.conf that at first returns ELOOP
+                    # (i.e. broken symlink) when opened, but on the
+                    # second call opens as usual.  We check that there
+                    # was an os.unlink() performed
+                    if args[0].startswith('/etc/resolv.conf'):
+                        self._resolv_unlinked = True
+                        mock_handle.__enter__ = mock.Mock(
+                            side_effect=[IOError(errno.ELOOP,
+                                                 os.strerror(errno.ELOOP),
+                                                 args[0]),
+                                         mock_handle])
                     self.file_handle_mocks[args[0]] = mock_handle
+
                 return mock_handle
             elif args[0].startswith('/sys/class/net'):
                 mock_args = [os.path.join(
@@ -143,6 +163,9 @@ class TestGlean(base.BaseTestCase):
         mock_call.return_value = 0
         self.useFixture(fixtures.MonkeyPatch('subprocess.call', mock_call))
 
+        mock_unlink = mock.Mock(return_value=0)
+        self.useFixture(fixtures.MonkeyPatch('os.unlink', mock_unlink))
+
         cmd.main()
 
         output_filename = '%s.%s.network.out' % (provider, distro.lower())
@@ -170,6 +193,9 @@ class TestGlean(base.BaseTestCase):
             write_handle = self.file_handle_mocks[dest].write
             write_handle.assert_called_once_with(content)
 
+        if self._resolv_unlinked:
+            mock_unlink.assert_called_once_with('/etc/resolv.conf')
+
         # Check hostname
         meta_data_path = 'mnt/config/openstack/latest/meta_data.json'
         hostname = None
@@ -187,13 +213,15 @@ class TestGlean(base.BaseTestCase):
                 [mock.call(hostname), mock.call('\n')])
 
         # Check hosts entry
-        calls = [mock.call('127.0.0.1 %s\n' % hostname), ]
+        hostname_ip = ips[provider]
+        calls = [mock.call('%s %s\n' % (hostname_ip, hostname)), ]
         short_hostname = hostname.split('.')[0]
         if hostname != short_hostname:
-            calls.append(mock.call('127.0.0.1 %s\n' % short_hostname))
+            calls.append(mock.call('%s %s\n' % (hostname_ip, short_hostname)))
 
         self.file_handle_mocks['/etc/hosts'].write.assert_has_calls(
             calls, any_order=True)
 
     def test_glean(self):
-        self._assert_distro_provider(self.distro, self.style)
+        with mock.patch('glean.systemlock.Lock'):
+            self._assert_distro_provider(self.distro, self.style)
